#include "mgos.h"

#ifdef MGOS_HAVE_DHT
#include "main.h"
#include "mgos_dht.h"
#include "mgos_config.h"
#include "mgos_prometheus_metrics.h"
#include "mqtt.h"

#define MAX_DHT 8

static struct mgos_dht *s_dht[MAX_DHT];
static int s_num_dht = 0;

static void print_chunk(struct mg_connection *nc, char *name, char *fmt, ...) {
  char chunk[500];
  int chunklen=0;
  va_list ap;

  snprintf(chunk, sizeof(chunk), "%s%s", name, fmt[0]=='{' ? "" : " ");
  va_start(ap, fmt);
  vsnprintf(chunk+strlen(chunk), sizeof(chunk)-strlen(chunk), fmt, ap);
  va_end(ap);
  strncat(chunk, "\n", sizeof(chunk));
  chunklen=strlen(chunk);
  LOG(LL_DEBUG, ("Chunk '%s' with length %d", chunk, chunklen));
  mg_printf(nc, "%X\r\n%s\r\n", chunklen, chunk);

}

static void dht_prometheus_metrics(struct mg_connection *nc, void *user_data) {
  int i;

  // BUG -- repeated HELP and TYPE makes Prometheus parser bork :(
  mgos_prometheus_metrics_printf(nc, GAUGE,
    "temperature", "Temperature in celcius",
    "{sensor=\"0\",type=\"DHT\"} %f", mgos_dht_get_temp(s_dht[0]));
  mgos_prometheus_metrics_printf(nc, GAUGE,
    "humidity", "Relative humidity percentage",
    "{sensor=\"0\",type=\"DHT\"} %f", mgos_dht_get_humidity(s_dht[0]));

  for (i=1; i<s_num_dht; i++) {
    print_chunk(nc, "temperature", "{sensor=\"%d\",type=\"DHT\"} %f", i, mgos_dht_get_temp(s_dht[i]));
    print_chunk(nc, "humidity", "{sensor=\"%d\",type=\"DHT\"} %f", i, mgos_dht_get_humidity(s_dht[i]));
  }

  (void) user_data;
}

static bool dht_create(int pin, enum dht_type type) {
  struct mgos_dht *dht;
  if (s_num_dht == MAX_DHT) {
    LOG(LL_ERROR, ("No more sensor slots available (%d added)", MAX_DHT));
    return false;
  }

  dht=mgos_dht_create(pin, type);
  if (!dht) {
    LOG(LL_ERROR, ("Could not create DHT sensor on pin %d", pin));
    return false;
  }
  s_dht[s_num_dht] = dht;
  s_num_dht++;

  return true;
}

void dht_init() {
  char *tok;

  memset(s_dht, 0, sizeof (struct mgos_dht *) * MAX_DHT);
  tok = strtok((char *)mgos_sys_config_get_app_dht_gpio(), ", ");
  while (tok) {
    int gpio;
    gpio = atoi(tok);
    tok=strtok(NULL, ", ");
    dht_create(gpio, AM2302);
    mgos_msleep(250);
  }

  if (s_num_dht>0)
    mgos_prometheus_metrics_add_handler(dht_prometheus_metrics, NULL);
}

#else
void dht_init() {
  LOG(LL_INFO, ("DHT disabled, include library in mos.yml to enable"));
}
#endif
